from xml.etree.ElementTree import ElementTree
from optparse import OptionParser
import os
import shutil

def main():
        p = OptionParser()
        p.add_option("--sourcePath", "-s", dest="dirnameNB", default="F:\\pubmed\\BMC Bioinformatics\\", help="Absolute path of the directory where articles are stored")
        p.add_option("--destPath", "-d", dest="destDirnameNB", default="F:\\Cleaned_Texts\\", help="Absolute path of the directory where the cleaned articles are stored")
        p.add_option("--copyDirectory", dest="copyDirnameNB", default="F:\\pubmed\\Articles%20Keywords\\", help="Absolute path of the directory where to copy the articles with keywords if needed")
        p.add_option("--dictionary", "-v", dest="dictionaryFileNameNB", default="F:\\@_Lorenzo\\@_Tesi\\@_Framework_Parts\\Dictionary_and_Stopwords\\gene_dictionary.txt", help="Absolute path of the file with the genes and proteins dictionary")
        p.add_option("--stopDictionary", "-p", dest="stopWordsFileNameNB", default="F:\\@_Lorenzo\\@_Tesi\\@_Framework_Parts\\Dictionary_and_Stopwords\\stopWords.txt", help="Absolute path of the file with the stopwords dictionary")        
        p.add_option("--statistics", "-t", dest="statFileNameNB", default="Text_Statistics.txt", help="Absolute path of the file where the final statistics are stored")
	p.add_option("--gnFoundFileName", dest="gnFoundFileNameNB", default="DocGenes.txt", help="Absolute path of the file where all the files wih genes (and the corresponding number of genes) are stored")
        p.add_option("--suffix", "-n", dest="suffixNB", default="cleaned", help="Suffix to be added to the processed article name")
        p.add_option("--copyArticles", dest="copyArticles", default="copy", help="Says whether to copy the articles with keywords in a specified directory")
	p.add_option("--alfa", dest="alfa", default="2.0", help="Value of the alfa parameter of the grading function")
	p.add_option("--beta", dest="beta", default="0.1", help="Value of the beta parameter of the grading function")
	p.add_option("--gamma", dest="gamma", default="0.1", help="Value of the gamma parameter of the grading function")
        p.add_option("--numberAndDuplicates", dest="numAndDupOrNot", default="N", help="If set to Y the files NUM and DUP with the number of distinct and duplicated genes will be produced. If not set nothing will be done")
        p.add_option("--linux", "-l", dest="linuxOrNot", default="", help="If not set -> working under Windows; if set to linux working under Linux")
        
        option, arguments = p.parse_args()

        if option.linuxOrNot == "linux" :
                dirname = option.dirnameNB.replace('%20', ' ')
		dirname = dirname.replace("F:\\", "/media/DATI/")
		dirname = dirname.replace('\\', '/')
		destDirname = option.destDirnameNB.replace('%20', ' ')
		destDirname = destDirname.replace("F:\\", "/media/DATI/")
		destDirname = destDirname.replace('\\', '/')
		copyDirname = option.copyDirnameNB.replace('%20', ' ')
		copyDirname = copyDirname.replace("F:\\", "/media/DATI/")
		copyDirname = copyDirname.replace('\\', '/')
		dictionaryFileName = option.dictionaryFileNameNB.replace('%20', ' ')
		dictionaryFileName = dictionaryFileName.replace("F:\\", "/media/DATI/")
		dictionaryFileName = dictionaryFileName.replace('\\', '/')		
		suffix = option.suffixNB.replace('%20', ' ')
		stopWordsFileName = option.stopWordsFileNameNB.replace('%20', ' ')
		stopWordsFileName = stopWordsFileName.replace("F:\\", "/media/DATI/")
		stopWordsFileName = stopWordsFileName.replace('\\', '/')
		statisticsFileName = option.statFileNameNB.replace('%20', ' ')		
		gnFoundFileName = option.gnFoundFileNameNB.replace('%20', ' ')		
		
	else :
                dirname = option.dirnameNB.replace('%20', ' ')
		destDirname = option.destDirnameNB.replace('%20', ' ')
		copyDirname = option.copyDirnameNB.replace('%20', ' ')		
		dictionaryFileName = option.dictionaryFileNameNB.replace('%20', ' ')
		suffix = option.suffixNB.replace('%20', ' ')
		stopWordsFileName = option.stopWordsFileNameNB.replace('%20', ' ')
        	statisticsFileName = option.statFileNameNB.replace('%20', ' ')
		gnFoundFileName = option.gnFoundFileNameNB.replace('%20', ' ')		
        	
        
        outputDirName = destDirname + os.sep
        if (os.path.exists(outputDirName)) :
                print "The specified output directory already exists: can't over-write. Returning"
                return
        else :
                os.mkdir(outputDirName)

        print 
	print "Alfa = ", option.alfa, "; Beta = ", option.beta, "; Gamma = ", option.gamma
        print 
        print "PHASE 1 TEXTS CLEANING"
        print

        print " --- List of the processed files --- "
        print
        i = 1
        counterDocKeywords = 0
        print dirname
        for fileName in os.listdir(dirname) :
                
                # Absolute path and name of the source file
                source = os.path.normpath(dirname+os.sep+fileName)
                print 'Source: ', source
                # Absolute path and name of the destination file
                ext = os.path.splitext( fileName )
                dest = outputDirName + ext[0].replace(' ','_') + "_" + suffix +".txt"
                dest = os.path.normpath(dest)
                
                #print str(i)+ ")"
                #print "Source: " + source
                #print "Destination: " + dest
                i += 1
                k = 0
                k = parserFunction(source,dest, option.numAndDupOrNot)
                if k == 1 :
                      counterDocKeywords+=1
                      if option.copyArticles == "copy" :
                              copyDest = os.path.normpath(copyDirname+os.sep+fileName)
                              shutil.copy(source,copyDest)
                
        print i-1, "texts cleaned"
        print counterDocKeywords, "documents have the keywords"

        print 
        print "PHASE 1 COMPLETED: TEXTS CLEANED"
        print

        print 
        print "PHASE 2: ADDING WEIGTHS"
        print

        sourceDirectory = os.path.normpath(outputDirName)
        vocabulary = os.path.normpath(dictionaryFileName)
        stopWord = os.path.normpath(stopWordsFileName)
        statistics = os.path.normpath(outputDirName + os.sep + statisticsFileName)
	gnFoundFile = os.path.normpath(outputDirName + os.sep + gnFoundFileName)
        weightsAdding(sourceDirectory,vocabulary,stopWord, statistics, gnFoundFile, float(option.alfa), float(option.beta), float(option.gamma), option.numAndDupOrNot) 

        print 
        print "PHASE 2 COMPLETED"
        print
        
        return


def parserFunction(sourceFile,destFile,numAndDupOrNot):
	
	if numAndDupOrNot == "Y":
		fstatistic = open ("statisticsDocument.txt", mode='a')        

        tree = ElementTree()
        found_title = 0
        try:
            tree.parse(sourceFile)
        except Exception, inst:
            print "Unexpected error opening %s: %s" % (sourceFile, inst)
            return

        f = open (destFile, mode='w')

        f.write ("<article>\n")
        keywordDict ={}
        i = 0
        numAbstract = 0

        frontElement = tree.find("front")

        if frontElement :

                frontIterator = frontElement.getiterator()
                for frontSubElement in frontIterator:
                        #Text that precedes all child elements (may be None)
                        #if frontSubElement.tag == "title-group" or frontSubElement.tag == "article-categories" or frontSubElement.tag == "abstract" or frontSubElement.tag == "kwd-group":
                        if frontSubElement.tag == "title-group" or frontSubElement.tag == "abstract" or frontSubElement.tag == "kwd-group":
                                if frontSubElement.tag == "title-group" :
                                        f.write("\n<title>\n")
                                if frontSubElement.tag == "abstract" and numAbstract == 0:
                                        numAbstract = 1
                                        f.write("\n<abstract>\n")
                                        
                                if frontSubElement.text:
                                        # if frontSubElement.tag == "article-categories"
                                        if frontSubElement.tag == "kwd-group":                                
						keywordDict[frontSubElement.text.encode('ascii','ignore')] = ''
                                                #keywordDict[frontSubElement.text.encode('latin-1', 'ignore')] = ''
                                        else :
						entireText = frontSubElement.text.encode('ascii','ignore')
                                                #entireText = frontSubElement.text.encode('latin-1', 'ignore')
                                                i = formattedPrint(entireText,f,i)
                                                
                                recursivePrint(frontSubElement, f, keywordDict, i)
                                
                                if frontSubElement.tag == "title-group" :
                                        f.write(".\n</title>\n")

                if numAbstract != 0 :			
                        f.write(".\n</abstract>\n")
		else :
			if numAndDupOrNot == "Y":	
				fstatistic.write(sourceFile + " No Abstract\n" )
        else :
		if numAndDupOrNot == "Y":
			fstatistic.write(sourceFile + " No front\n" )
                #f.write ("\n--------No front-----------\n")
		

        
        f.write("\n<body>\n")       
        bodyElement = tree.find("body")
        if bodyElement :

                bodyIterator = bodyElement.getiterator()
                for bodySubElement in bodyIterator:
                        
                        #Text that precedes all child elements (may be None)
                        if bodySubElement.tag == "title" or bodySubElement.tag == "p" :
                                if bodySubElement.tag == "title" :
                                                i =0
                                                f.write("\n")                      
                                if bodySubElement.text:
                                        
					entireText = bodySubElement.text.encode('ascii','ignore')
                                        #entireText = bodySubElement.text.encode('latin-1', 'ignore')
                                        i = formattedPrint(entireText,f,i)
                                
                                recursivePrint(bodySubElement, f, keywordDict, i)
                                if bodySubElement.tag == "title" :
                                                i = 0
                                                f.write("\n")                        
                                #f.write("\n")
        else :                
                #f.write ("\n--------No Body-----------\n")                        
		if numAndDupOrNot == "Y":
			fstatistic.write(sourceFile + " No Body\n" )
        f.write(".\n</body>\n")

  
        backElement = tree.find("back")
        #print backElement.tag
        if backElement :
                backIterator = backElement.getiterator()
                for backSubElement in backIterator:
                        #Text that precedes all child elements (may be None)
                        if backSubElement.tag == "glossary" :                        
                                if backSubElement.text:
                                        entireText = backSubElement.text.encode('ascii','ignore')
                                        #entireText = backSubElement.text.encode('latin-1', 'ignore')
                                        i = formattedPrint(entireText,f,i)
                                        
                                recursivePrint(backSubElement, f, keywordDict,i)
                                #f.write("\n")
        else :
##                f.write ("\n--------No Back-----------\n")                        
		if numAndDupOrNot == "Y":
			fstatistic.write(sourceFile + " No Back\n" )
                        
        f.write("\n<keywords>\n")
        i = 0
        for kwd in keywordDict.keys():
                i += 1
                f.write(kwd +"\n")                
        f.write("\n</keywords>\n")
        keywords = 0
        if i >0 :
                keywords = 1
        else :
                keywords = 0
        f.close()

	if numAndDupOrNot == "Y":
		fstatistic.close()

        return keywords

def recursivePrint(elemento, f, keywordDict, i) :
        if elemento.getchildren():
                #Can also use: "for child in element.getchildren():"
                for child in elemento:
                        if child.tag != "math" and child.tag != "table":
                                
                                if child.tag == "title" :
                                        f.write("\n")
                                        i = 0
                                if child.text :
                                        #if elemento.tag == "article-categories" or child.tag == "subject" or child.tag == "term" or
                                        if elemento.tag == "kwd" or child.tag == "kwd":  
                                                keywordDict[child.text.encode('ascii','ignore')] = ''                                              
                                                #keywordDict[child.text.encode('latin-1', 'ignore')] = ''
                                        else :
						entireText = child.text.encode('ascii','ignore')
                                                #entireText = child.text.encode('latin-1', 'ignore')
                                                i = formattedPrint(entireText,f,i)
                                                
                                #The "tail" on each child element consists of the text
                                #that comes after it in the parent element content, but
                                #before its next sibling.
                                if child.tail:
					entireText = child.tail.encode('ascii','ignore')
                                        #entireText = child.tail.encode('latin-1', 'ignore')
                                        i = formattedPrint(entireText,f,i)                                        
                                        
                                if child.tag == "title" :
                                        i = 0
                                        f.write(".\n")
                                #print "Call rec I'm " + child.tag
                                recursivePrint(child,f, keywordDict, i)
                                
                        else :
                                if child.tail:
					entireText = child.tail.encode('ascii','ignore')
                                        #entireText = child.tail.encode('latin-1', 'ignore')
                                        i = formattedPrint(entireText,f,i)
        return

def formattedPrint(entireText,f,i):
        tot = i
        for word in entireText.strip().split() :
                tot += len(word)+1
                if (tot <= 80) :
                        f.write(word+" ")                                                                
                else :
                        tot =   len(word)+1
                        f.write("\n"+word+" ")
        return tot

def weightsAdding(sourceDirectory,vocabulary, stopWord, statistics, gnFoundFile , alfa, beta, gamma,numAndDupOrNot):

        print 
        print "PHASE 2.1: Filling the vocabulary"
        print

        f = open (stopWord, mode='r')
        

        stopDict ={}
        
        for line in f :
                words = line.strip()
                stopDict[words] = ''

        f.close()
        
        f = open (vocabulary, mode='r')
        

        geneDict ={}
        
        for line in f :
                word = line.strip()
                if ( (word.strip().isdigit() == False) and (stopDict.has_key(word.strip()) == 0) and (word[0].isdigit() == False) and (word[0] != '#') and (word[0] != '-')) :
                        geneDict[word.strip()] = ''
                

        f.close()

        print 
        print "PHASE 2.1 ENDED: vocabulary filled"
        print

        print 
        print "PHASE 2.2: adding the weights to each file"
        print

        foundGenes = {}
	
	noBreakList = {}
	# List of words that don't cause sentence termination
	noBreakList["U.S."]=''; noBreakList["[U.S."]='' ; noBreakList["(U.S."]=''	
	noBreakList["St."]=''; noBreakList["[St."]='' ;	noBreakList["(St."]='' ; noBreakList["-St."]=''	
	noBreakList["Rush-Presbyterian-St."]=''		
	noBreakList["Dr."]='' ;	noBreakList["[Dr."]='' ; noBreakList["(Dr."]=''	
	noBreakList["r.p.m."]='' ; noBreakList["(r.p.m."]='' ; noBreakList["[r.p.m."]=''		
	noBreakList["p.p.m."]='' ; noBreakList["(p.p.m."]='' ; noBreakList["[p.p.m."]=''		
	noBreakList["Jan."]=''	; noBreakList["[Jan."]='' ; noBreakList["(Jan."]='' 
	noBreakList["Feb."]='' ; noBreakList["(Feb."]='' ; noBreakList["[Feb."]=''
	noBreakList["Mar."]='' ; noBreakList["(Mar."]='' ; noBreakList["[Mar."]=''
	#noBreakList["April."]=''	
	noBreakList["May."]='' ; noBreakList["(May."]='' ; noBreakList["[May."]=''	
	noBreakList["Jun."]='' ; noBreakList["(Jun."]='' ; noBreakList["[Jun."]=''
	noBreakList["C-Jun."]='' ; noBreakList["c-Jun."]=''
	noBreakList["Jul."]='' ; noBreakList["(Jul."]='' ; noBreakList["[Jul."]=''
	noBreakList["Aug."]='' ; noBreakList["[Aug."]='' ; noBreakList["(Aug."]=''
	#noBreakList["September."]=''
	noBreakList["Oct."]='' ; noBreakList["[Oct."]='' ; noBreakList["(Oct."]=''
	noBreakList["Nov."]='' ; noBreakList["[Nov."]='' ; noBreakList["(Nov."]=''	
	noBreakList["Dec."]='' ; noBreakList["[Dec."]='' ; noBreakList["(Dec."]=''		
	noBreakList["a.m."]='' ; noBreakList["(a.m."]='' ; noBreakList["[a.m."]=''		
	noBreakList["p.m."]='' ; noBreakList["(p.m."]='' ; noBreakList["[p.m."]=''			
	noBreakList["Mrs."]='' ; noBreakList["(Mrs."]='' ; noBreakList["[Mrs."]=''
	noBreakList["Rep."]='' ; noBreakList["'Rep."]='' ; noBreakList["(Rep."]=''; noBreakList["('Rep."]='' ; noBreakList["[Rep."]=''
	noBreakList["c.p.m."]='' ; noBreakList["(c.p.m."]='' ; noBreakList["[c.p.m."]='' ; noBreakList["C.p.m."]='' ; noBreakList["(C.p.m."]=''	; noBreakList["[C.p.m."]='' ; noBreakList["stimulation/C.p.m."]=''						

        grandTotal = 0
        fgenes = open (gnFoundFile, mode='w')
	print alfa, " ", beta, " ", gamma
        for fileName in os.listdir(sourceDirectory) :
                
                # Absolute path and name of the source file
                source = os.path.normpath(sourceDirectory+os.sep+fileName)                
                weights = [[],[],[]]
		geneNumbers = [[],[],[]]
		geneDuplicates = [[],[],[]]


		# Splitting in sentences, counting the number of genes for each sentence, and giving 
		# weigths to each sentence (1.0 if no genes, 2.0 + (0.1 * num of genes) if genes are found)
		gnCount = 0
                gnCount = geneCounter(source,weights,geneDict, foundGenes, noBreakList, alfa, beta, gamma, geneNumbers, geneDuplicates)
                if (gnCount > 0) :
			fgenes.write( source+" "+str(gnCount)+ "\n")	# Recording all files with genes in a file		
                grandTotal += gnCount
		
		# Modifiy the source file by adding the xml tags related to the weights
                f = open (source, mode='a')
                f.write ("\n<weights>\n")
                for i in range(0,len(weights)) :
                        if i == 0 :
                                if (len(weights[i]) != 0) :
                                        f.write ("\n<TitleWeights>\n")
                                        for item in weights[i] :
                                                f.write (str(item)+" ")                                
                                        f.write ("\n</TitleWeights>\n")
                        if i == 1 :
                                if (len(weights[i]) != 0) :
                                        f.write ("\n<AbstractWeights>\n")
                                        for item in weights[i] :
                                                f.write (str(item)+" ")                                
                                        f.write ("\n</AbstractWeights>\n")
                        if i == 2 :
                                if (len(weights[i]) != 0) :
                                        f.write ("\n<BodyWeights>\n")                                        
                                        for item in weights[i] :
                                                f.write (str(item)+" ")
                                        f.write ("\n</BodyWeights>\n")
                                
                f.write ("\n</weights>\n")
		if numAndDupOrNot == "Y":
		        f.write ("\n<numbers>\n")
		        for i in range(0,len(geneNumbers)) :
		                if i == 0 :
		                        if (len(geneNumbers[i]) != 0) :
		                                f.write ("\n<TitleNumbers>\n")
		                                for item in geneNumbers[i] :
		                                        f.write (str(item)+" ")                                
		                                f.write ("\n</TitleNumbers>\n")
		                if i == 1 :
		                        if (len(geneNumbers[i]) != 0) :
		                                f.write ("\n<AbstractNumbers>\n")
		                                for item in geneNumbers[i] :
		                                        f.write (str(item)+" ")                                
		                                f.write ("\n</AbstractNumbers>\n")
		                if i == 2 :
		                        if (len(geneNumbers[i]) != 0) :
		                                f.write ("\n<BodyNumbers>\n")                                        
		                                for item in geneNumbers[i] :
		                                        f.write (str(item)+" ")
		                                f.write ("\n</BodyNumbers>\n")
		                        
		        f.write ("\n</numbers>\n")                

		        f.write ("\n<duplicates>\n")
		        for i in range(0,len(geneDuplicates)) :
		                if i == 0 :
		                        if (len(geneDuplicates[i]) != 0) :
		                                f.write ("\n<TitleDuplicates>\n")
		                                for item in geneDuplicates[i] :
		                                        f.write (str(item)+" ")                                
		                                f.write ("\n</TitleDuplicates>\n")
		                if i == 1 :
		                        if (len(geneDuplicates[i]) != 0) :
		                                f.write ("\n<AbstractDuplicates>\n")
		                                for item in geneDuplicates[i] :
		                                        f.write (str(item)+" ")                                
		                                f.write ("\n</AbstractDuplicates>\n")
		                if i == 2 :
		                        if (len(geneDuplicates[i]) != 0) :
		                                f.write ("\n<BodyDuplicates>\n")                                        
		                                for item in geneDuplicates[i] :
		                                        f.write (str(item)+" ")
		                                f.write ("\n</BodyDuplicates>\n")
		                        
		        f.write ("\n</duplicates>\n")              
                f.write ("\n</article>\n")                
                f.close()

	fgenes.close()

        print 
        print "PHASE 2.2 ENDED: weights added to each file"
        print

        print 
        print "PHASE 2.3: writing the statistics file"
        print

        f = open (statistics, mode='w')
        counter = 0
        frequentGenes = {}	
        for entry in foundGenes.keys():
                if foundGenes[entry] >= 10 :
                        ##f.write(entry +"    "+str(foundGenes[entry])+" FREQUENT\n")
                        frequentGenes[entry] = foundGenes[entry]                        
                else :
                        f.write(entry +"    "+str(foundGenes[entry])+"\n")
                counter+=1

        f.write("\n ----- FREQUENT GENES (more than 10 matches) -----\n")
        for entry in frequentGenes.keys():
                f.write(entry +"    "+str(foundGenes[entry])+"\n")

        f.write("\n\n ----- FINAL STATISTICS -----\n\n")
        f.write(" - Number of distinct entries (genes) found in the texts: "+str(counter) +"\n")
        f.write(" - Grand Total (total number of occurrences): "+str(grandTotal) +"\n")

        
        f.close()

        print 
        print "PHASE 2.3 ENDED: statistics file written"
        print
                
        return

def geneCounter (source,weights,geneDict,foundGenes, noBreakList, alfa, beta, gamma, geneNumbers, geneDuplicates) :

        f = open (source, mode='r')
	#fw = open (source+"DEBUGDEBUG", mode='w')
        #print source

        t = 0 ; a = 0 ; b = 0
	geneInSentence = {} #Dictionary for the genes found in a sentence
	geneNumber = [0] ; totCounter = [0]; sentenceEnd = [0]; duplicatedGenes = [0]

        for line in f :
                if line.strip() == "<title>" :
                        title = []
			titleGeneNum = []
			titleGeneDup = []
                        t = 1
                        continue
                if line.strip() == "</title>" :
                        if sentenceEnd[0] != 1 :
				#fw.write("<spanAdded>")
                                title.append(giveScore(geneNumber[0],duplicatedGenes[0], alfa, beta, gamma))
				titleGeneNum.append(geneNumber[0])
				titleGeneDup.append(duplicatedGenes[0])							
                                geneNumber[0] = 0
				duplicatedGenes[0] = 0
				geneInSentence.clear()
                        weights[0]=title
			geneNumbers[0]= titleGeneNum
			geneDuplicates[0] = titleGeneDup		
                        sentenceEnd[0] = 0
                        t = 0
                        continue
                if line.strip() == "<body>" :
                        body = []
			bodyGeneNum = []
			bodyGeneDup = []
                        b = 1
                        continue
                if line.strip() == "</body>" :
                        if sentenceEnd[0] != 1 :
				#fw.write("<spanAdded>")
                                body.append(giveScore(geneNumber[0],duplicatedGenes[0], alfa, beta, gamma))				
				bodyGeneNum.append(geneNumber[0])
				bodyGeneDup.append(duplicatedGenes[0])			
                                geneNumber[0] = 0
				duplicatedGenes[0] = 0
				geneInSentence.clear()
                        weights[2]=body
			geneNumbers[2]= bodyGeneNum
			geneDuplicates[2] = bodyGeneDup		
                        sentenceEnd[0] = 0
                        b = 0
                        continue
                if line.strip() == "<abstract>" :
                        abstract = []
			abstractGeneNum = []
			abstractGeneDup = []
                        a = 1
                        continue
                if line.strip() == "</abstract>" :
                        if sentenceEnd[0] != 1 :
				#fw.write("<spanAdded>")
                                abstract.append(giveScore(geneNumber[0],duplicatedGenes[0], alfa, beta, gamma))
				abstractGeneNum.append(geneNumber[0])
				abstractGeneDup.append(duplicatedGenes[0])			
                                geneNumber[0] = 0
				duplicatedGenes[0] = 0
				geneInSentence.clear()
                        weights[1]=abstract
			geneNumbers[1]= abstractGeneNum
			geneDuplicates[1] = abstractGeneDup		
                        sentenceEnd[0] = 0
                        a = 0
                        continue
                
                if t == 1 :
                	#wordAnalyzer (line, noBreakList, geneDict, foundGenes, totCounter, geneNumber, sentenceEnd, title,titleGeneNum,titleGeneDup, alfa, beta, gamma, geneInSentence,duplicatedGenes,fw) 
			wordAnalyzer (line, noBreakList, geneDict, foundGenes, totCounter, geneNumber, sentenceEnd, title,titleGeneNum,titleGeneDup, alfa, beta, gamma, geneInSentence,duplicatedGenes) 
                if a == 1 :
                	#wordAnalyzer (line, noBreakList, geneDict, foundGenes, totCounter, geneNumber, sentenceEnd, abstract,abstractGeneNum,abstractGeneDup, alfa, beta, gamma,geneInSentence,duplicatedGenes, fw)                                                
                	wordAnalyzer (line, noBreakList, geneDict, foundGenes, totCounter, geneNumber, sentenceEnd, abstract,abstractGeneNum,abstractGeneDup, alfa, beta, gamma, geneInSentence,duplicatedGenes)                                                
                if b == 1 :
                	#wordAnalyzer (line, noBreakList, geneDict, foundGenes, totCounter, geneNumber, sentenceEnd, body,bodyGeneNum, bodyGeneDup, alfa, beta, gamma,geneInSentence,duplicatedGenes, fw)
                	wordAnalyzer (line, noBreakList, geneDict, foundGenes, totCounter, geneNumber, sentenceEnd, body,bodyGeneNum, bodyGeneDup, alfa, beta, gamma, geneInSentence,duplicatedGenes)                                                

        f.close()        
	#fw.close()
        return totCounter[0] 

#def wordAnalyzer (line, noBreakList, geneDict, foundGenes, totCounter, geneNumber, sentenceEnd, block,blockGeneNum,blockGeneDup, alfa, beta, gamma,geneInSentence,duplicatedGenes, fw) : 
def wordAnalyzer (line, noBreakList, geneDict, foundGenes, totCounter, geneNumber, sentenceEnd, block,blockGeneNum,blockGeneDup, alfa, beta, gamma,geneInSentence,duplicatedGenes) : 
	# Split the line into the words that compose it
	for word in line.strip().split() :
		#fw.write(word+" ")		
		#Analyze if the word is a gene				
		wordFound = [""]
		if ( geneOrNot(geneDict,word,wordFound) == 1 ) :
			if wordFound[0] == "":
				print "Empty!"
				os._exit(1)
			#print wordFound[0]
			geneStatistics (geneDict, word, foundGenes)
			totCounter[0] += 1
			if ( geneInSentence.has_key(wordFound[0]) ):
				# Duplicated Gene in the Sentence
				duplicatedGenes[0] += 1		
				geneInSentence[wordFound[0]] += 1		
			else :
				# Newly found gene in the Sentence
				geneNumber[0] += 1
				geneInSentence[wordFound[0]] = 1
		
		# Analyze if a sentence is over
		if ( (noBreakList.has_key(word) == False) and (word.endswith("!") or word.endswith("?") or word.endswith(";") or word.endswith(".") or word.endswith("|") or word.endswith("?\"") or word.endswith("!\"") or word.endswith(".\"") ) ):
			# End of the sentence
			#fw.write("<span>\n")
			block.append(giveScore(geneNumber[0],duplicatedGenes[0], alfa, beta, gamma))
			blockGeneNum.append(geneNumber[0])
			blockGeneDup.append(duplicatedGenes[0])			
			geneNumber[0] = 0
			duplicatedGenes[0] = 0
			geneInSentence.clear()
			sentenceEnd[0] = 1
		else :
			sentenceEnd[0] = 0			
	#fw.write("\n")
	return

def giveScore(geneNumber,duplicatedGenes, alfa, beta, gamma):
        
        if geneNumber == 0:
                score = 1.0
        else :
                score = alfa + geneNumber*beta + duplicatedGenes*gamma
                
        return score

def geneOrNot (geneDict,word,wordFound):
	
	word = word.replace('(', ''); word = word.replace(')', '') 
	word = word.replace('[', ''); word = word.replace(']', '')

	if ( geneDict.has_key(word) ) :
		wordFound[0] = word
		return 1

	if ( word.endswith("!") or word.endswith("?") or word.endswith(";") or word.endswith(".") or word.endswith("|") or word.endswith(",") ):
		if (geneDict.has_key(word[:-1]) ) :
			wordFound[0] = word[:-1]
			return 1
	if ( word.endswith("?\"") or word.endswith("!\"") or word.endswith(".\"") or word.endswith("?)") or word.endswith("!)") or word.endswith(".)") or word.endswith(")?") or word.endswith(")!") or word.endswith(").") or word.endswith(");") or word.endswith("):") or word.endswith(")-") ):
		if (geneDict.has_key(word[:-2]) ) :
			wordFound[0] = word[:-2]
			return 1
	if ( word.startswith("!") or word.startswith("?") or word.startswith(";") or word.startswith(".") or word.startswith("|") or word.startswith(",")):
		if (geneDict.has_key(word[1:]) ) :
			wordFound[0] = word[1:]
			return 1
	if ( word.startswith("?\"") or word.startswith("!\"") or word.startswith(".\"") or word.startswith("?)") or word.startswith("!)") or word.startswith(".)") or word.startswith(")?") or word.startswith(")!") or word.startswith(").") or word.startswith(");") or word.startswith("):") or word.startswith(")-") ) :
		if (geneDict.has_key(word[2:]) ) :
			wordFound[0] = word[2:]
			return 1

	return 0

def geneStatistics (geneDict, word, foundGenes) :

	word = word.replace('(', ''); word = word.replace(')', '') 
	word = word.replace('[', ''); word = word.replace(']', '')	        

	wordGeneName = word

	if ( word.endswith("!") or word.endswith("?") or word.endswith(";") or word.endswith(".") or word.endswith("|") or word.endswith(",") ):
		if (geneDict.has_key(word[:-1]) ) :
			wordGeneName = word[:-1]
	if ( word.endswith("?\"") or word.endswith("!\"") or word.endswith(".\"") or word.endswith("?)") or word.endswith("!)") or word.endswith(".)") or word.endswith(")?") or word.endswith(")!") or word.endswith(").") or word.endswith(");") or word.endswith("):") or word.endswith(")-") ):
		if (geneDict.has_key(word[:-2]) ) :
			wordGeneName = word[:-2]
	if ( word.startswith("!") or word.startswith("?") or word.startswith(";") or word.startswith(".") or word.startswith("|") or word.startswith(",")):
		if (geneDict.has_key(word[1:]) ) :
			wordGeneName = word[1:]
	if ( word.startswith("?\"") or word.startswith("!\"") or word.startswith(".\"") or word.startswith("?)") or word.startswith("!)") or word.startswith(".)") or word.startswith(")?") or word.startswith(")!") or word.startswith(").") or word.startswith(");") or word.startswith("):") or word.startswith(")-") ) :
		if (geneDict.has_key(word[2:]) ) :
			wordGeneName = word[2:]

        if foundGenes.has_key(wordGeneName) :
                foundGenes[wordGeneName]+=1
        else :
                foundGenes[wordGeneName] = 1
        return
                

if __name__ == "__main__":
	# Someone is launching this directly
	main()
