import sys
from Bio import Entrez
from Bio import Medline
from Bio.EUtils.ThinClient import ThinClient
import urllib
import threading
import bibtexparse

try:  
    import gtk  
    import gtk.glade  
except:  
    print("GTK Not Availible")
    sys.exit(1)

from xml.sax import make_parser

from xml.sax.handler import ContentHandler 



Entrez.email = "tryit@example.com"

class PubMedRetrieve:

    def __init__( self , wTree, progressbar, settings=[]):
    	self.wTree = wTree
    	self.settings = settings
	self.stats = []
	self.records = {}
	self.pb = progressbar


    def getStats(self):
    	return self.stats
    	
    def PMCSearch(self, articlelist, elNum):
    	'''
    	This function searches via PubMed Central for the given quey
    	'''

	settings = self.settings
	self.articlelist = articlelist
	records = {}
	journal = self.wTree.get_widget('journalKey').get_text()
	
    	if settings[1] != "":
    		# Creates the query to give in input to PMC, considering date,
    		# number of results and journal
    		if journal == '':
	    		if settings[2] == 2:
    				terms = settings[1]
		    	if settings[2] == 0:
		    		terms = settings[1] + ' AND ((\"' + settings[3] + '\"[PDat]:\"' + settings[4] + '\"[PDat]))'
		    	elif settings[2] == 1:
		    		terms = settings[1] + ' AND (\"last ' + settings[3] + ' days\"[PDat])'
	    	else:
	    		if settings[2] == 2:
    				terms = settings[1] + ' AND (\"' + journal + '[Journal])'
		    	if settings[2] == 0:
		    		terms = settings[1] + ' AND (\"' + journal + '[Journal]) AND ((\"' + settings[3] + '\"[PDat]:\"' + settings[4] + '\"[PDat]))'
		    	elif settings[2] == 1:
		    		terms = settings[1] + ' AND (\"' + journal + '[Journal]) AND (\"last ' + settings[3] + ' days\"[PDat])'
		
		# PMC search using Entrez tools    		
		handle = Entrez.esearch(db="pmc", term=terms + ' AND ((\"open access\"[filter]))', retmax=settings[0])
		record = Entrez.read(handle)
	        s = record["IdList"]
	        nrecord = int(record["Count"])
		count = 0
		notConsidered = 0
		l = dbids(s)
		recordSumm = ThinClient().esummary_using_dbids(dbids = l).read()
		# Save summary in a temporary xml file
		f = open('./PubMedCentral/temp.xml','w')
		f.write(recordSumm)
		f.close()
		# Read temporary file to parse results and calls xmlSummaryParser,
		# that will display those results in the articles list
		f = open('./PubMedCentral/temp.xml','r')
	        parser = make_parser()
	        pmcHandler = xmlSummaryParser(settings[0], articlelist, self.wTree, s, elNum)		      
	        parser.setContentHandler(pmcHandler)
		parser.parse(f)
		f.close()

	        st, self.records, elNum = pmcHandler.getinfo()
	        stats = [record["Count"], st[0], st[1]]

		return (stats, self.records, elNum)


    def downloadPMC(self, idlist):
    	'''
    	This function is called during the download phase.
    	It's goal is to download selected articles from a PMC
    	search, parse and save them in pseudo-xml format
    	'''
	settings = self.settings
	recordH = Entrez.efetch(db="pmc", id = idlist)
	f = open('./PubMedCentral/temp2.xml','w')
	f.write(recordH.read())
	f.close()       
	f = open('./PubMedCentral/temp2.xml','r')
	parser = make_parser()
	pmcHandler = xmlPMCparser(settings[0], self.articlelist, self.wTree, idlist, self.pb)
	parser.setContentHandler(pmcHandler)		
	parser.parse(f)
	f.close()
	return pmcHandler.getinfo()
    
    
    def PubMedSearch(self, articlelist, elNum):
    	'''
    	Searches via PubMed.
    	This function also retrieves articles abstract, that means that download
    	phase for PM will only require to copysuch abstract (along with all article
    	infos) in a pseudo-xml format, with no need to download anything.
    	'''
	settings = self.settings
	records = {}

	# Gets input query and journal
    	entry = self.wTree.get_widget("entryKey")
    	self.articlelist = articlelist
	journal = self.wTree.get_widget('journalKey').get_text()
	
	if entry.get_text() != "":
    		# Sets the right query with date and journal, and do the search
    		if journal == '':
	    		if settings[2] == 2:
	    			handle = Entrez.esearch(db='pubmed', term=settings[1], retmax=settings[0]*2)
	    		elif settings[2] == 0:
				handle = Entrez.esearch(db="pubmed",term=settings[1],
		    					retmax=settings[0]*2, mindate = str(settings[3]),
		    					maxdate= str(settings[4]))
		    	else:
		    		handle = Entrez.esearch(db="pubmed",term=settings[1],
		    					retmax=settings[0]*2, reldate = settings[3])
		else:
			if settings[2] == 2:
	    			handle = Entrez.esearch(db='pubmed', term=settings[1] + ' AND (\"' + journal + '[Journal])', retmax=settings[0]*2)
	    		elif settings[2] == 0:
				handle = Entrez.esearch(db="pubmed",term=settings[1] + ' AND (\"' + journal + '[Journal])',
		    					retmax=settings[0]*2, mindate = str(settings[3]),
		    					maxdate= str(settings[4]))
		    	else:
		    		handle = Entrez.esearch(db="pubmed",term=settings[1] + ' AND (\"' + journal + '[Journal])',
		    					retmax=settings[0]*2, reldate = settings[3])
		# Read search results, and save them in a variable (records)
		record = Entrez.read(handle)
	        s = record["IdList"]
	        nrecord = int(record["Count"])
	        recordH = Entrez.efetch(db="pubmed", id = s, rettype="medline", retmode="text")

	        sRecordH = Medline.parse(recordH)

		count = 0
		notConsidered = 0

		
		for record in sRecordH:
			#print record
	        	if "TI" in record:
	        		if "AB" in record and "AU" in record and "JT" in record and "CRDT" in record:
	        			elNum+=1
	        			filename = record["PMID"]
	        			records[count] = {'id':record['PMID'], 'title':record['TI'].replace('&','&amp;').replace('>','&gt;').replace('<','&lt;'), 'author': record['AU'], 'journal':record['JT'].replace('&','&amp;').replace('>','&gt;').replace('<','&lt;'), 'year': record['CRDT'], 'abstract':record['AB'].replace('&','&amp;').replace('>','&gt;').replace('<','&lt;')}
	        			# Populate articles list
	        			self.articlelist.append([ True, record["TI"], record["AU"], record["JT"], record["CRDT"], record["PMID"], '#FFFFF0', 'PM', elNum])

		    			count += 1
					if count >= settings[0]:
		    				break
		    		else:
		    		# Some results do not have abstract. We decided to ignore them.
		    			notConsidered += 1
		    	else:
				self.articlelist.append([None,"Your search did not match any document!",'','','','','','',elNum])
	        
	        stats=[nrecord, notConsidered, count]
	        return(stats, records, elNum)	


class xmlSummaryParser(ContentHandler):
	'''
	Parses the results from the PMC esummary, and saves them in
	a variable containing all infos about the articles returned
	from the web search
	'''
	def __init__(self, retmax, articlelist, wTree, s, elNum):
		self.wTree = wTree
		self.articlelist = articlelist
		self.retmax = retmax
		self.idList = s
		self.records = {}
		self.elNum = elNum
		self.title, self.author, self.journal, self.date, self.kwd = '','','','',''
		self.count, self.countValid, self.articleID = 0, 0, ''
		self.isTitle, self.isAuthor, self.isJournal, self.isDate, self.isArticleID, self.isKwd = 0,0,0,0,0,0
		
	def startElement(self, name, attrs):
		if name == 'Id':
			self.isArticleID = 1
			self.count += 1
			self.elNum += 1
		elif name == 'Item':
			name = attrs.get('Name', None)
			if name == "Author":
				self.isAuthor = 1
			elif name == 'Title':
				self.isTitle = 1
			elif name == 'PubDate':
				self.isDate = 1
			elif name == 'Journal':
				self.isJournal = 1
				self.countValid += 1

			
	def characters (self, ch):
		if self.isArticleID:
			self.articleID = ch
			self.isArticleID = 0
		elif self.isTitle:
			self.title = ch
			self.isTitle = 0
		elif self.isAuthor:
			self.author = ch
			self.isAuthor = 0
		elif self.isDate:
			self.date = ch
			self.isDate = 0
		elif self.isJournal:
			self.journal = ch
			self.isJournal = 0
			self.records[self.count] = {'id':self.articleID, 'title':self.title, 'author': self.author, 'journal':self.journal, 'year': self.date, 'abstract':''}
			self.articlelist.append([ True, self.title, self.author, self.journal, self.date, self.articleID, '#FFFFF0', 'PMC', self.elNum])
		
	def getinfo(self):
		return [self.count-self.countValid, self.countValid], self.records, self.elNum
	

class xmlPMCparser(ContentHandler):
	'''
	This class parses the full texts XML files, that comes from the
	download phase using PubMed Central. It gets articles informations
	and full text.
	'''
	def __init__(self, retmax, articlelist, wTree, s, pb):
		self.wTree = wTree
		self.articlelist = articlelist
		self.retmax = retmax
		self.pb = pb
		self.idList = s
		self.artText, self.absText = {}, {}
		self.text, self.articleName = '', 'temp'
		self.articleText = ''
		self.kwdList = {}
		self.kwd = []
		self.title, self.author, self.journal, self.year, self.month, self.abstract = '','','','','',''
		self.artbody, self.count, self.countValid, self.articleTitle, self.at = 0, 0, 0, 0, 0
		self.isAuthor, self.isJournal, self.isYear, self.isMonth, self.isAbstract, self.isKwd = 0,0,0,0,0,0
		
	def startElement(self, name, attrs):

		if name == 'article-id' and attrs.get('pub-id-type', None) == 'pmc':
			self.articleTitle = 1
		elif name == 'abstract' and self.countValid < self.retmax:
			self.isAbstract = 1
			self.abstract = ''
		elif name == 'kwd-group':
			self.isKwd = 1
			self.kwd = []
		elif name == 'body' and self.countValid < self.retmax:
			self.text = ''
			self.artbody = 1
			
			
	def endElement(self, name):
		if self.countValid < self.retmax:
			if name == 'article':
				self.count += 1
			elif name == 'abstract':
				self.isAbstract = 0
			elif name == 'kwd-group':
				self.isKwd = 0
				self.kwdList[self.articleName] = self.kwd
			elif name == 'body':
				self.artText[self.articleName] = self.text
				self.absText[self.articleName] = self.abstract
				self.countValid += 1
				self.artbody = 0
		
			
	def characters (self, ch):
		if self.artbody:
			self.text += ch + ' '
		if self.articleTitle:
			self.articleName = ch
			self.articleTitle = 0
		if self.isKwd:
			self.kwd.append(ch)
		if self.isAbstract:
			self.abstract += ch + ' '
	
			
	def getinfo(self):
		for el in self.artText:
			self.artText[el] = self.artText[el].replace('&','&amp;').replace('<','&lt;')
		for el in self.absText:
			self.absText[el] = self.absText[el].replace('&','&amp;').replace('<','&lt;')
		return  self.artText, self.absText, self.kwdList
		
class dbids:
	def __init__(self, ids):
		self.db='pmc'
		self.ids = ids
	
