import httplib
import urllib #, htmllib
import HTMLParser, urlparse
import re
import gtk

from BeautifulSoup import BeautifulSoup #, SoupStrainer
from downloadText import downloadText
import bibtexparse


class GoogleScholarSearch:
	"""
	@brief This class searches Google Scholar (http://scholar.google.com)

	Search for articles and publications containing terms of interest.
	"""
	def __init__(self):
		"""
		@brief Empty constructor.
		"""
		self.SEARCH_HOST = "scholar.google.com"
		self.SEARCH_BASE_URL = "/scholar"
		

	def search(self, wTree, settings, articlelist, elNum):
		"""
		@brief This function searches Google Scholar using the specified terms.
		@param settings, array with date, max number of records
		@return stats, array with records found, processed, and not considered
		"""
		
		alfa = 2
		count = 0
		start = 0
		npag = 0
		totalFound = 9999999
		notConsidered = 0
		records = {}
		
		while count < settings[0] and start <= totalFound and start < 1000-settings[0]*alfa:
			pageCount = 0
			params = urllib.urlencode({
						   'q': settings[1],
						   'num': settings[0]*alfa,
						   'as_ylo': settings[3],
						   'as_yhi': settings[4],
						   'start' : start,
						   'as_publication' : settings[5]
						 })
			headers = {'User-Agent': 'Mozilla/4.0 (compatible; MSIE 5.5; Windows NT)'}
	
			url = self.SEARCH_BASE_URL+"?"+params
			conn = httplib.HTTPConnection(self.SEARCH_HOST)
			conn.request("GET", url, {}, headers)

			resp = conn.getresponse()
	
			if resp.status==200:
				html = resp.read()
				results = []
				html = html.decode('ascii', 'ignore')

				# Correct errors on Google Scholar result page
				html = html.replace("\"o" , "\" o")
				html = html.replace(" color=>",">")
				bs = BeautifulSoup(html)

				p = LinksParser(wTree)
				
				# Find page results and total results
				index = bs.find(text=re.compile("Results"))
				if index == None:
					return
				index = index.findNext('b')
				minV = str(index).replace('<b>','')
				minV = str(minV).replace('</b>','')
				index = index.findNext('b')
				maxV = str(index).replace('<b>','')
				maxV = str(maxV).replace('</b>','')
				recordInPage = int(maxV) - int(minV) + 1
				
				# If this is the first page, I look for total number of results
				if minV == '1':
					index = index.findNext('b')
					totalFound = str(index).replace('<b>','')
					totalFound = totalFound.replace('</b>','')
					totalFound = totalFound.replace(',','')
					totalFound = int(totalFound)
					
				spans = bs.find(text=re.compile("x25ba"))
				
				# Populate articles list analyzing Google Scholar web page results
				while spans != None and count < settings[0]:
					title = spans.findPrevious('a')
					temp = spans.findNext('a')
					AJY = spans.findNext('span')
					s = str(temp).split('href=\"')
					s1 = s[1].split('\" onm')
					s2 = s1[0].split('=')
					if len(s2) == 1 and str(s2).find('ref') == -1:
	    					p.feed(str(temp))
	    					p.feed(str(title))
	    					p.feed(str(AJY))
    						count += 1
    						pageCount += 1
    						info, articleName = p.get_infos()
    						fileName = info[3].split('/')
    						fileName = fileName[len(fileName)-1]
    						if not 'pdf' in fileName:
    							fileName += '.html' 
    						elNum += 1
    						records[count] = {'id':count+1, 'title':articleName, 'author': info[0], 								'journal':info[1], 'year': info[2], 'link':info[3]}

    						articlelist.append([ True, records[count]['title'], records[count]['author'], records[count]['journal'], records[count]['year'], fileName , '#FFFFF0', 'GS', elNum])
					spans = temp.findNext(text=re.compile("x25ba"))
				notConsidered += recordInPage - pageCount
				start += settings[0]*alfa
				npag += 1

			else:
				tB = wTree.get_widget("textview1").get_buffer()
				tB.set_text("ERROR: ", resp.status, resp.reason)
				wTree.get_widget("textview1").set_buffer(tB)
				return [0, 0, elNum], ''
		stats=[totalFound, notConsidered, count]
		return stats, records, elNum



class LinksParser(HTMLParser.HTMLParser):
    '''
    This class parses parts of the Google Scholar results
    HTML page, to get, when available, infos about a
    specific article (title, authors, date, journal, ...)
    '''
    def __init__(self, wTree):
        HTMLParser.HTMLParser.__init__(self)
        self.seen = {}
        self.wTree = wTree
        self.listaddr, self.infos = [], []
        self.isValid, self.validurl, self.isAJY, self.test = True, False, False, True
        self.index = True
        self.articleName, self.info, self.tempArticleName = '', '', ''
        
    def handle_starttag(self, tag, attributes):
        if tag == 'a':
        	for name, value in attributes:
        		if name == 'class' and value == 'fl':
        	    		self.isValid = False
        	    	if name == 'href' and value not in self.seen:
        	    	    	self.seen[value] = True
        	    	    	pieces = urlparse.urlparse(value)
        	    	   	if pieces[0] != 'http': return
        	    	    	if self.test: 
        	    	    		self.addr = urlparse.urlunparse(pieces)
        	    	    		self.test = False
        	    	    	else: self.test = True
        	    	    	self.listaddr = [self.listaddr, self.addr]
				self.infos = [None, None, None, self.addr]
        	    	    	return
	elif tag =='span':
		for name, value in attributes:
			if name == 'class' and value == 'a':
				self.isAJY = True
	else: return

    def handle_endtag(self, tag):
    	if tag == 'a':
    		self.validurl = False
    		self.isValid = True
    		if self.tempArticleName != '':
    			self.articleName = self.tempArticleName
    		self.tempArticleName = ''
    		return
    	elif tag == 'span':
    		temp = ['', '']
    		temp2 = ['', '']
    		self.isAJY = False
    		temp = self.info.split(' - ')
    		if len(temp) == 1:
    			temp[0] = ''
    			temp2 = ['', '']
    		elif len(temp) == 2:
    			temp2 = temp[0].split(',')
    			if re.match("[0-9]+$", temp2[1]) or re.match("[0-9]+$", temp2[0]):
    				temp[0] = ''
    			else: temp2 = ['', '']
    		else: temp2 = temp[1].split(', ')
    		if len(temp2) == 1:
    			if re.match("[0-9]+$", temp2[0]):
	    			temp2 = ['', temp2[0]]
	    		else: temp2 = [temp2[0], '']
	    	if len(temp2) == 3:
	    		temp2 = [temp2[0] + temp2[1], temp2[2]]
    		self.infos = [temp[0], temp2[0], temp2[1], self.infos[3]]
    		self.info = ''
    		return
    	else: return

    def handle_data(self, data):
	if self.isAJY:
		self.info += data
	elif self.isValid:
		self.tempArticleName += data

    def get_infos(self):
    	return self.infos, self.articleName
    	
    	
